import subprocess
import os
import json
import shutil

# Define file paths
source_video = r"D:\WorkSpaceLaptop\UnityPackFF\eustacestv-video128.ogv"
target_video = r"D:\WorkSpaceLaptop\UnityPackFF\girlfriend.mp4"
output_video = r"D:\WorkSpaceLaptop\UnityPackFF\output_256.ogv"

# Function to find FFmpeg & FFprobe dynamically
def find_ffmpeg():
    ffmpeg_path = shutil.which("ffmpeg")
    ffprobe_path = shutil.which("ffprobe")

    if not ffmpeg_path or not ffprobe_path:
        common_paths = [
            r"C:\\ffmpeg\\bin\\ffmpeg.exe",
            r"C:\\Users\\wifis\\AppData\\Local\\Programs\\Shotcut\\ffmpeg.exe",
            r"C:\\Program Files (x86)\\FFmpeg\\bin\\ffmpeg.exe",
        ]

        for path in common_paths:
            if os.path.exists(path):
                ffmpeg_path = path
                ffprobe_path = path.replace("ffmpeg.exe", "ffprobe.exe")
                break

    if not ffmpeg_path or not ffprobe_path:
        raise FileNotFoundError("FFmpeg and FFprobe were not found. Please install FFmpeg and ensure it's in your PATH.")

    return ffmpeg_path, ffprobe_path

# Load FFmpeg paths
ffmpeg_path, ffprobe_path = find_ffmpeg()

# Function to extract video properties
def get_video_properties(video_path):
    cmd = [
        ffprobe_path, "-v", "error",
        "-select_streams", "v:0",
        "-show_entries", "stream=width,height,r_frame_rate,bit_rate",
        "-show_entries", "format=duration,size",
        "-of", "json", video_path
    ]
    result = subprocess.run(cmd, capture_output=True, text=True)
    return json.loads(result.stdout)

# Extract properties from the source video
video_info = get_video_properties(source_video)

# Get resolution, frame rate, duration, and file size
width = video_info["streams"][0]["width"]
height = video_info["streams"][0]["height"]
frame_rate = eval(video_info["streams"][0]["r_frame_rate"])  # Convert to float
duration = float(video_info["format"]["duration"])
file_size = int(video_info["format"]["size"])

# Handle missing bitrate by setting a default value
bit_rate = video_info["streams"][0].get("bit_rate", "2000000")  # Default: 2 Mbps if missing

# Trim target video to exactly 1 minute 12 seconds (72 seconds)
trim_length = 72  # seconds
hours = int(trim_length // 3600)
minutes = int((trim_length % 3600) // 60)
seconds = int(trim_length % 60)
trim_time = f"{hours:02}:{minutes:02}:{seconds:02}"

# Convert target video to match source properties with better quality
ffmpeg_command = [
    ffmpeg_path, "-i", target_video,
    "-vf", "scale=256:256",
    "-r", "60",
    "-fps_mode", "cfr",
    "-q:v", "1",
    "-preset", "slow",
    "-c:v", "libtheora",
    "-to", trim_time,
    "-y", output_video
]

subprocess.run(ffmpeg_command)

# Only check if file exists (skip size adjustment that degrades quality)
if not os.path.exists(output_video):
    raise FileNotFoundError(f"FFmpeg failed to create the output file: {output_video}")

print(f"Conversion complete! Output file: {output_video}")
