#!/usr/bin/env python3

import sys
import io
from unitypack.asset import Asset

def replace_movie_texture(asset_bundle_path, new_movie_path, output_path):
    """ Replaces an existing MovieTexture's OGV data in the asset bundle. """

    # Load the asset bundle
    with open(asset_bundle_path, 'rb') as f:
        asset_data = f.read()

    buf = io.BytesIO(asset_data)
    buf.name = asset_bundle_path  # Ensure UnityPack reads it correctly
    asset = Asset.from_file(buf)

    # Locate the MovieTexture object
    movie_texture = None
    for path_id, obj in asset.objects.items():
        if path_id == 2 and obj.type == "MovieTexture":
            movie_texture = obj.read()
            print(f"Found MovieTexture at path_id: {path_id}")
            break

    if movie_texture is None:
        print("Error: No MovieTexture found. Exiting.")
        return

    # Print attributes for debugging
    print(f"MovieTexture attributes before replacement: {dir(movie_texture)}")

    # Load the new movie data
    with open(new_movie_path, 'rb') as f:
        new_movie_data = f.read()

    # Ensure we replace the correct attribute
    if hasattr(movie_texture, "movie_data"):
        movie_texture.movie_data = bytearray(new_movie_data)  # Correct attribute!
        print("Updated movie_data successfully.")
    else:
        print("Error: movie_data attribute not found. Cannot replace movie data.")
        return

    # Force reassignment into the asset structure
    asset.objects[2]._contents = movie_texture

    print(f"Replaced MovieTexture at path_id 2 with new data ({len(new_movie_data)} bytes).")

    # Save the modified asset bundle
    with open(output_path, 'wb') as outf:
        asset.save(outf)

    print(f"Modified asset bundle saved as {output_path}")

if __name__ == '__main__':
    if len(sys.argv) != 4:
        print("Usage: replace_ogv_final_fixed.py input.assetbundle new_movie.ogv output.assetbundle", file=sys.stderr)
        sys.exit(1)

    replace_movie_texture(sys.argv[1], sys.argv[2], sys.argv[3])
